import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { FormControl, ReactiveFormsModule } from "@angular/forms";
import { RouterModule } from "@angular/router";
import { ConsentComponent } from "../../consent/consent.component.js";
import { ErrorComponent } from "../../error/error.component.js";
import { CheckboxComponent } from "../../inputs/checkbox/checkbox.component.js";
import { MultiLineComponent } from "../../inputs/multi-line/multi-line.component.js";
import { SingleLineComponent } from "../../inputs/single-line/single-line.component.js";
import { SuccessComponent } from "../../success/success.component.js";
import { UserinfoComponent } from "../../userinfo/userinfo.component.js";
import { ApiService } from "../../api.service.js";

/**
 *
 */
@Component({
  imports: [
    RouterModule,
    CommonModule,
    ConsentComponent,
    UserinfoComponent,
    ErrorComponent,
    ReactiveFormsModule,
    CheckboxComponent,
    SingleLineComponent,
    MultiLineComponent,
    SuccessComponent,
  ],
  selector:    'event-form',
  styleUrl:    './event.component.css',
  templateUrl: "./event.component.html",
})
export class EventComponent {
  public loading = false;
  public error = "";
  public success = false;

  public firstName = new FormControl("");
  public lastName = new FormControl("");
  public email = new FormControl("");
  public company = new FormControl("");

  public eventDescription = new FormControl("");
  public eventTopic = new FormControl("");
  public eventLocation = new FormControl("");
  public eventDate = new FormControl("");
  public eventBudget = new FormControl("");
  public travelCovered = new FormControl(false);
  public lodgingCovered = new FormControl(false);
  public foodCovered = new FormControl(false);

  public consent = new FormControl(false);

  /**
   * @param apiService
   */
  constructor(private readonly apiService: ApiService) {}

  /**
   * @param e
   */
  public submit(e: MouseEvent): void {
    this.error = "";
    const { form } = e.target as HTMLButtonElement;
    const valid = form?.reportValidity();
    if (!valid) {
      return;
    }
    this.loading = true;

    this.apiService.
      submitEvent({
        companyName:      this.company.value ?? undefined,
        email:            this.email.value ?? undefined,
        consent: this.consent.value ?? false,
        eventBudget:      this.eventBudget.value ?? undefined,
        eventDate:        this.eventDate.value ?? undefined,
        eventDescription: this.eventDescription.value ?? undefined,
        eventLocation:    this.eventLocation.value ?? undefined,
        eventTopic:       this.eventTopic.value ?? undefined,
        firstName:        this.firstName.value ?? undefined,
        foodCovered:      this.foodCovered.value ?? false,
        lastName:         this.lastName.value ?? undefined,
        lodgingCovered:   this.lodgingCovered.value ?? false,
        travelCovered:    this.travelCovered.value ?? false,
      }).
      then((res) => {
        if ("error" in res) {
          this.error = res.error;
          this.loading = false;
        } else {
          this.error = "";
          this.success = true;
        }
      }).
      catch(() => {
        this.error = "An error occurred while submitting the form.";
        this.loading = false;
      });
  }
}
