import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { FormControl, ReactiveFormsModule } from "@angular/forms";
import { RouterModule } from "@angular/router";
import type { Platform } from "@repo/types/prod/unions/platform.js";
import { ApiService } from "../../api.service.js";
import { ConsentComponent } from "../../consent/consent.component.js";
import { ErrorComponent } from "../../error/error.component.js";
import { CheckboxComponent } from "../../inputs/checkbox/checkbox.component.js";
import { MultiLineComponent } from "../../inputs/multi-line/multi-line.component.js";
import { SelectMenuComponent } from "../../inputs/select-menu/select-menu.component.js";
import { SingleLineComponent } from "../../inputs/single-line/single-line.component.js";
import { SuccessComponent } from "../../success/success.component.js";
import { UserinfoComponent } from "../../userinfo/userinfo.component.js";

/**
 *
 */
@Component({
  imports: [
    RouterModule,
    CommonModule,
    ConsentComponent,
    UserinfoComponent,
    ErrorComponent,
    ReactiveFormsModule,
    CheckboxComponent,
    SingleLineComponent,
    SelectMenuComponent,
    MultiLineComponent,
    SuccessComponent,
  ],
  selector:    'staff-form',
  styleUrl:    './staff.component.css',
  templateUrl: "./staff.component.html",
})
export class StaffComponent {
  public loading = false;
  public error = "";
  public success = false;

  public firstName = new FormControl("");
  public lastName = new FormControl("");
  public email = new FormControl("");
  public company = new FormControl("");

  public understandVolunteer = new FormControl(false);
  public platform = new FormControl<Platform>("Forum");
  public platformUsername = new FormControl("");
  public whyJoin = new FormControl("");
  public currentBehaviour = new FormControl("");
  public priorExperience = new FormControl("");
  public internalConflict = new FormControl("");
  public handlingTrauma = new FormControl("");
  public difficultSituation = new FormControl("");
  public leadershipSituation = new FormControl("");

  public consent = new FormControl(false);

  /**
   * @param apiService
   */
  constructor(private readonly apiService: ApiService) {}

  /**
   * @param e
   */
  public submit(e: MouseEvent): void {
    this.error = "";
    const { form } = e.target as HTMLButtonElement;
    const valid = form?.reportValidity();
    if (!valid) {
      return;
    }
    this.loading = true;

    this.apiService.
      submitStaff({
        currentBehaviour:    this.currentBehaviour.value ?? undefined,
        email:               this.email.value ?? undefined,
        difficultSituation:  this.difficultSituation.value ?? undefined,
        firstName:           this.firstName.value ?? undefined,
        consent:             this.consent.value ?? false,
        internalConflict:    this.internalConflict.value ?? undefined,
        handlingTrauma:      this.handlingTrauma.value ?? undefined,
        lastName:            this.lastName.value ?? undefined,
        leadershipSituation: this.leadershipSituation.value ?? undefined,
        platform:            this.platform.value ?? undefined,
        platformUsername:    this.platformUsername.value ?? undefined,
        priorExperience:     this.priorExperience.value ?? undefined,
        understandVolunteer: this.understandVolunteer.value ?? false,
        whyJoin:             this.whyJoin.value ?? undefined,
      }).
      then((res) => {
        if ("error" in res) {
          this.error = res.error;
          this.loading = false;
        } else {
          this.error = "";
          this.success = true;
        }
      }).
      catch(() => {
        this.error = "An error occurred while submitting the form.";
        this.loading = false;
      });
  }
}
